PALETTE_INIT( multipac );
VIDEO_START( multipac );
VIDEO_UPDATE( multipac );
VIDEO_UPDATE( superabc );
WRITE8_HANDLER( multipac_gfxbank_w );
WRITE8_HANDLER( multipac_tilecolor_w );
UINT8 speedcheat;
INTERRUPT_GEN( pacman_interrupt );
WRITE8_HANDLER( pacman_leds_w );
WRITE8_HANDLER( pacman_coin_counter_w );
WRITE8_HANDLER( pacman_coin_lockout_global_w );
WRITE8_HANDLER( pacman_interrupt_vector_w );
